/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.talk

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.common.network.ApiCallback
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.partner.talk.model.*
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.talk.model.Friends
import com.kakao.sdk.talk.model.Order
import com.kakao.sdk.template.DefaultTemplate

/**
 * @suppress
 */
val TalkApiClient.partnerApi: PartnerTalkApi
    get() = ApiFactory.kapiWithOAuth.create(PartnerTalkApi::class.java)

/**
 * 사용자의 카카오 플랫폼 내 친구 목록을 가져옵니다.
 *
 * @param friendType [FriendType]
 * @param friendFilter [FriendFilter]
 * @param friendOrder [FriendOrder]
 */
@JvmOverloads
fun TalkApiClient.friendsForPartner(
    friendType: FriendType? = null,
    friendFilter: FriendFilter? = null,
    friendOrder: FriendOrder? = null,
    secureResource: Boolean? = true,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null,
    callback: (friends: Friends<PartnerFriend>?, error: Throwable?) -> Unit
) {
    partnerApi.friends(friendType, friendFilter, friendOrder, offset, limit, order, secureResource)
        .enqueue(object: ApiCallback<Friends<PartnerFriend>>() {
            override fun onComplete(model: Friends<PartnerFriend>?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 카카오톡 채팅방 목록을 가져옵니다.
 */
@JvmOverloads
fun TalkApiClient.chatList(
    filters: List<ChatFilter>? = null,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null,
    secureResource: Boolean? = true,
    callback: (chats: Chats?, error: Throwable?) -> Unit
) {
    partnerApi.chatList(filters?.joinToString(","), offset, limit, order, secureResource)
        .enqueue(object: ApiCallback<Chats>() {
            override fun onComplete(model: Chats?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 사용자의 카카오톡 채팅방에 속한 멤버를 조회합니다. 채팅방 아이디를 기준으로 조회하며 친구인 멤버만 조회할지 여부를 선택할 수 있습니다.
 */
@JvmOverloads
fun TalkApiClient.chatMembers(
    chatId: Long,
    friendsOnly: Boolean? = null,
    includeProfile: Boolean? = null,
    secureResource: Boolean? = true,
    offset: Int? = null,
    limit: Int? = null,
    order: Order? = null,
    callback: (chatMembers: ChatMembers?, error: Throwable?) -> Unit
) {
    partnerApi.chatMembers(chatId, friendsOnly, includeProfile, offset, limit, order, secureResource)
        .enqueue(object: ApiCallback<ChatMembers>() {
            override fun onComplete(model: ChatMembers?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 개발자사이트에서 생성한 메시지 템플릿을 사용하여, 조회한 친구를 대상으로 카카오톡으로 메시지를 전송합니다.
 *
 * 템플릿을 생성하는 방법은 https://developers.kakao.com/docs/template 을 참고하시기 바랍니다.
 */
@JvmOverloads
fun TalkApiClient.sendCustomMessageForPartner(
    receiverUuids: List<String>,
    templateId: Long,
    templateArgs: Map<String, String>? = null,
    callback: (messageSendResult: PartnerMessageSendResult?, error: Throwable?) -> Unit
) {
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), templateId, templateArgs)
        .enqueue(object: ApiCallback<PartnerMessageSendResult>() {
            override fun onComplete(model: PartnerMessageSendResult?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 개발자사이트에서 생성한 메시지 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 *
 * 템플릿을 생성하는 방법은 https://developers.kakao.com/docs/template 을 참고하시기 바랍니다.
 */
@JvmOverloads
fun TalkApiClient.sendCustomMessageToChats(
    receiverChatIds: List<Long>,
    templateId: Long,
    templateArgs: Map<String, String>? = null,
    callback: (messageSendResult: PartnerMessageSendResult?, error: Throwable?) -> Unit
) {
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds), templateId, templateArgs)
        .enqueue(object: ApiCallback<PartnerMessageSendResult>() {
            override fun onComplete(model: PartnerMessageSendResult?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 기본 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 */
fun TalkApiClient.sendDefaultMessageForPartner(
    receiverUuids: List<String>,
    template: DefaultTemplate,
    callback: (messageSendResult: PartnerMessageSendResult?, error: Throwable?) -> Unit
) {
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), template)
        .enqueue(object: ApiCallback<PartnerMessageSendResult>() {
            override fun onComplete(model: PartnerMessageSendResult?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 기본 템플릿을 사용하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 */
fun TalkApiClient.sendDefaultMessageToChats(
    receiverChatIds: List<Long>,
    template: DefaultTemplate,
    callback: (messageSendResult: PartnerMessageSendResult?, error: Throwable?) -> Unit
) {
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds), template)
        .enqueue(object: ApiCallback<PartnerMessageSendResult>() {
            override fun onComplete(model: PartnerMessageSendResult?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 지정된 URL을 스크랩하여, 조회한 친구를 대상으로 카카오톡으로 메시지를 전송합니다.
 *
 * 스크랩 커스텀 템플릿 가이드를 참고하여 템플릿을 직접 만들고 스크랩 메시지 전송에 이용할 수도 있습니다.
 *
 * @see RxTalkApiClient.friendsForPartner
 */
@JvmOverloads
fun TalkApiClient.sendScrapMessageForPartner(
    receiverUuids: List<String>,
    requestUrl: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null,
    callback: (messageSendResult: PartnerMessageSendResult?, error: Throwable?) -> Unit
) {
    partnerApi.sendMessage("uuid", KakaoJson.toJson(receiverUuids), requestUrl, templateId, templateArgs)
        .enqueue(object: ApiCallback<PartnerMessageSendResult>() {
            override fun onComplete(model: PartnerMessageSendResult?, error: Throwable?) {
                callback(model, error)
            }
        })
}

/**
 * 지정된 URL 을 스크랩하여, 특정 채팅방에 카카오톡 메시지를 전송합니다.
 *
 * 스크랩 커스텀 템플릿 가이드를 참고하여 템플릿을 직접 만들고 스크랩 메시지 전송에 이용할 수도 있습니다.
 */
@JvmOverloads
fun TalkApiClient.sendScrapMessageToChats(
    receiverChatIds: List<Long>,
    requestUrl: String,
    templateId: Long? = null,
    templateArgs: Map<String, String>? = null,
    callback: (messageSendResult: PartnerMessageSendResult?, error: Throwable?) -> Unit
) {
    partnerApi.sendMessage("chat_id", KakaoJson.toJson(receiverChatIds), requestUrl, templateId, templateArgs)
        .enqueue(object: ApiCallback<PartnerMessageSendResult>() {
            override fun onComplete(model: PartnerMessageSendResult?, error: Throwable?) {
                callback(model, error)
            }
        })
}
