/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.talk.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import kotlinx.android.parcel.Parcelize

/**
 * 파트너용으로 제공되는 친구 정보입니다.
 *
 * @property id 사용자 아이디. [registeredToApp]이 false 이면 null 이 됩니다.
 * @property uuid 메시지를 전송하기 위한 고유 아이디
 * @property profileNickname 친구의 대표 프로필 닉네임. 앱 가입친구의 경우 앱에서 설정한 닉네임. 미가입친구의 경우 톡 또는 스토리의 닉네임
 * @property profileThumbnailImage 친구의 썸네일 이미지 URL
 * @property favorite 즐겨찾기 추가 여부
 * @property serviceUserId
 * @property talkOs 톡에 가입된 기기의 os 정보 (android / ios)
 * @property msgAllowed 메시지 수신이 허용되었는지 여부. 앱가입 친구의 경우는 feed msg 에 해당. 앱미가입친구는 invite msg 에 해당
 *
 * @since 2.0.0
 */
@Parcelize
data class PartnerFriend(
    val id: Long?,
    val uuid: String,
    val profileNickname: String,
    val profileThumbnailImage: String,
    val favorite: Boolean,
    val serviceUserId: Long?,
    @SerializedName("app_registered") val registeredToApp: Boolean?,
    val relation: FriendRelation?,
    val talkOs: String?,
    @SerializedName("allowed_msg") val msgAllowed: Boolean?
) : Parcelable

/**
 * 친구 목록 필터링
 */
enum class FriendFilter {
    @SerializedName("none")
    NONE,
    @SerializedName("registered")
    REGISTERED,
    @SerializedName("invitable")
    INVITABLE;
}

/**
 * 친구 목록 정렬 기준.
 */
enum class FriendOrder {
    /**
     * 이름 순 정렬
     */
    @SerializedName("nickname")
    NICKNAME,
    /**
     * 나이 순 정렬
     */
    @SerializedName("age")
    AGE,
    /**
     * 즐겨찾기 순 정렬
     */
    @SerializedName("favorite")
    FAVORITE;
}

/**
 * 친구를 가져올 서비스 선택
 */
enum class FriendType {
    /**
     * 카카오톡 친구
     */
    @SerializedName("talk")
    KAKAO_TALK,
    /**
     * 카카오스토리 친구
     */
    @SerializedName("story")
    KAKAO_STORY,
    /**
     * 카카오톡과 카카오스토리 친구
     */
    @SerializedName("talkstory")
    KAKAO_TALK_AND_STORY,
    @UnknownValue
    UNKNOWN;
}

@Parcelize
data class FriendRelation(val talk: Relation, val story: Relation) : Parcelable

enum class Relation {
    @SerializedName("N/A")
    NONE,
    @SerializedName("friend")
    FRIEND,
    @SerializedName("no_friend")
    NOT_FRIEND,
    @UnknownValue
    UNKNOWN;
}